import { currMouseAction, backToEdit } from "../menutools.js"
import { IC_IMG, gate, colorCollision } from "../simulator.js";
import { MouseAction, IC_type } from "./Enums.js";
import { colorMouseOver, fileManager } from "../simulator.js";
import { checkCollision } from "../simulator.js";

/**
 * Prototype of integrated circuit
 * @classdesc Generic integrated circuit
 * @memberof module:IntegratedCiruits
 */
export class Integrated {

    

    /**
     * @constructor
     * @param {IC_type} type Integrated circuit type
     */
    constructor(type) {
        this.type = type;
        this.width = IC_IMG[this.type].width;
        this.height = IC_IMG[this.type].height;
        this.posX = mouseX - (this.width / 2);
        this.posY = mouseY - (this.height / 2);
        this.isSpawned = false;
        this.collided = false;
        this.offsetMouseX = 0;
        this.offsetMouseY = 0;
        this.isMoving = false;
        this.isSaved = false;
    }

    /**
     * Draw an Integrated class instance
     */
    draw() {

        if (!this.isSpawned) {
            this.posX = mouseX - (this.width / 2);
            this.posY = mouseY - (this.height / 2);
        }else if(!this.isSaved){
            fileManager.saveState();
            this.isSaved = true;
        }

        if (this.isMoving) {
            this.posX = mouseX + this.offsetMouseX;
            this.posY = mouseY + this.offsetMouseY;
        }
        
        if(this.isMoving || !this.isSpawned){
            //If is moving or is being spawed ceck collision
            this.collided = checkCollision(this);
        }

        image(IC_IMG[this.type], this.posX, this.posY);
        this.drawBoundaries();
    }

    /**
     * Draw element doundaries
     */
    drawBoundaries(){
        if(!this.collided && this.isMouseOver()){
            noFill();
            strokeWeight(2);
            stroke(colorMouseOver[0], colorMouseOver[1], colorMouseOver[2]);
            rect(this.posX, this.posY, IC_IMG[this.type].width, IC_IMG[this.type].height);
        }else if(this.collided){
            noFill();
            strokeWeight(2);
            stroke(colorCollision[0], colorCollision[1], colorCollision[2]);
            rect(this.posX, this.posY, IC_IMG[this.type].width, IC_IMG[this.type].height);
        }
    }

    /**
     * Checking if mouse is over this element
     * @returns {Boolean} Boolean value
     */
    isMouseOver() {
        if (mouseX > this.posX && mouseX < (this.posX + this.width)
            && mouseY > this.posY && mouseY < (this.posY + this.height))
            return true;
        return false;
    }

    /**
     * Called when mouse is pressed
     */
    mousePressed() {
        if (!this.isSpawned && !checkCollision(this)) {
            //If this element is not spawned
            //Spawn this on mouse position, return to edit mode
            this.posX = mouseX - (this.width / 2);
            this.posY = mouseY - (this.height / 2);
            this.isSpawned = true;
            backToEdit();
            return;
        }

        if (this.isMouseOver() || currMouseAction == MouseAction.MOVE) {
            //If mouse is over this element OR current mouse action is move
            //Set this isMoving to true and move this element to mouse position
            this.isMoving = true;
            this.offsetMouseX = this.posX - mouseX;
            this.offsetMouseY = this.posY - mouseY;
            this.posXOld = this.posX;
            this.posYOld = this.posY;
        }

        if(this.isMouseOver()){
            this.collided = checkCollision(this);
        }
    }

    /**
     * Called when mouse is clicked
     */
    mouseClicked()
    {
        // virtual
    }

    /**
     * Called when mouse is released:
     */
    mouseReleased() {  
        if(this.isMoving && (this.collided=checkCollision(this))){
            this.posX = this.posXOld;
            this.posY = this.posYOld
        } 
        this.isMoving = false;         
    }

    

    /**
     * Check if this element has collision with Coordinate
     * @param {Int[]} coordinate Coordinate to check collision
     * @returns {Boolean} Boolean value
     */
    checkCollision(coordinate){
        if((coordinate[0] + coordinate[2] < this.posX || coordinate[0] > this.posX + this.width || coordinate[1] + coordinate[3] < this.posY || coordinate[1] > this.posY + this.height)){
            return false;
        }
        return true;
    }

    /**
     * Return element position and dimension
     * @returns {Array} Array with [ posx , posy , widht , height]
     */
    getCoordinate(){
        let coordinate = [this.posX,this.posY,this.width,this.height];
        return coordinate
    }

}